@@ -38,7 +38,7 @@
 #define BE_INT_BUF_LEN (32 * BE_KB)	//Max length of a SysEx message for Elektroid
 #define BE_DEV_RING_BUF_LEN (256 * BE_KB)
 #define BE_DEVICE_NAME "hw:%d,%d,%d"
-#define BE_TMP_BUFF_LEN 256
+#define BE_TMP_BUFF_LEN (64 * BE_KB)	//This size is required by RtMidi as it needs enough space for the messages.
 
 #define BE_FILE_ICON_WAVE "elektroid-wave-symbolic"
 #define BE_FILE_ICON_SEQ "elektroid-sequence-symbolic"
@@ -48,7 +48,7 @@
 #define BE_DUMP_TIMEOUT 5000	//With and E-Mu ESI-2000 it takes this more than 3 seconds to receive to receive some packets after the process has started.
 #define BE_REST_TIME_US 50000
 #define BE_SYSEX_TIMEOUT_MS 5000
-#define BE_SYSEX_TIMEOUT_GUESS_MS 500	//When the request is not implemented, 5 s is too much.
+#define BE_SYSEX_TIMEOUT_GUESS_MS 1000	//When the request is not implemented, 5 s is too much.
 #define BE_SAMPLE_ID_NAME_SEPARATOR ":"
 
 #define BE_COMPANY_LEN 3
@@ -128,7 +128,9 @@ gint backend_init (struct backend *, const gchar *);
 
 void backend_destroy (struct backend *);
 
-ssize_t backend_tx_raw (struct backend *, const guint8 *, guint);
+ssize_t backend_rx_raw (struct backend *, guint8 *, guint);
+
+ssize_t backend_tx_raw (struct backend *, guint8 *, guint);
 
 gint backend_tx_sysex_no_status (struct backend *, struct sysex_transfer *);
 
@@ -166,6 +168,13 @@ void backend_destroy_data (struct backend *);
 
 gint backend_program_change (struct backend *, guint8, guint8);
 
+gint backend_send_controller (struct backend *backend, guint8 channel,
+			      guint8 controller, guint8 value);
+
+gint backend_send_rpn (struct backend *backend, guint8 channel,
+		       guint8 controller_msb, guint8 controller_lsb,
+		       guint8 value_msb, guint8 value_lsb);
+
 void backend_midi_handshake (struct backend *);
 
 /**
@@ -174,4 +183,6 @@ void backend_midi_handshake (struct backend *);
 
 const gchar *backend_strerror (struct backend *backend, gint error);
 
+enum path_type path_type_from_backend (struct backend *);
+
 #endif
