@@ -19,7 +19,6 @@
  */
 
 #include <gtk/gtk.h>
-#include <libgen.h>
 #include "utils.h"
 #include "notifier.h"
 
@@ -70,6 +69,16 @@ struct browser
   GMutex mutex;
   gboolean loading;
   gboolean dirty;
+  //Menu
+  GtkWidget *transfer_menuitem;
+  GtkWidget *play_separator;
+  GtkWidget *play_menuitem;
+  GtkWidget *options_separator;
+  GtkWidget *open_menuitem;
+  GtkWidget *show_menuitem;
+  GtkWidget *actions_separator;
+  GtkWidget *rename_menuitem;
+  GtkWidget *delete_menuitem;
 };
 
 void browser_set_item (GtkTreeModel *, GtkTreeIter *, struct item *);
