@@ -20,18 +20,15 @@
 
 #include <stdio.h>
 #include <math.h>
-#include <endian.h>
-#include <poll.h>
 #include <zlib.h>
-#include <libgen.h>
-#include "elektron.h"
 #include "elektron.h"
 #include "utils.h"
 #include "sample.h"
 #include "package.h"
+#include "common.h"
 #include "../config.h"
 
-#define DEVICES_FILE "/elektron-devices.json"
+#define DEVICES_FILE "/elektron/devices.json"
 
 #define DEV_TAG_ID "id"
 #define DEV_TAG_NAME "name"
@@ -56,7 +53,7 @@ static const gchar *FS_TYPE_NAMES[] = { "+Drive", "RAM" };
 #define ELEKTRON_NAME_MAX_LEN 32
 
 #define ELEKTRON_SAMPLE_INFO_PAD_I32_LEN 10
-#define ELEKTRON_LOOP_TYPE 0x7f
+#define ELEKTRON_LOOP_TYPE 0x7f000000
 
 struct elektron_sample_header
 {
@@ -198,12 +195,7 @@ static gchar *elektron_get_dev_and_fs_ext (struct backend *,
 
 static gchar *elektron_get_upload_path_smplrw (struct backend *,
 					       const struct fs_operations *,
-					       const gchar *, const gchar *,
-					       gint32 *);
-static gchar *elektron_get_upload_path_data (struct backend *,
-					     const struct fs_operations *,
-					     const gchar *, const gchar *,
-					     gint32 *);
+					       const gchar *, const gchar *);
 static gchar *elektron_get_download_path (struct backend *,
 					  const struct fs_operations *,
 					  const gchar *, const gchar *,
@@ -229,6 +221,8 @@ static void elektron_print_data (struct item_iterator *, struct backend *,
 
 static gchar *elektron_get_id_as_slot (struct item *, struct backend *);
 
+static gboolean elektron_sample_file_exists (struct backend *, const gchar *);
+static gboolean elektron_raw_file_exists (struct backend *, const gchar *);
 
 static const guint8 MSG_HEADER[] = { 0xf0, 0, 0x20, 0x3c, 0x10, 0 };
 
@@ -242,6 +236,7 @@ static const guint8 FS_SAMPLE_CREATE_DIR_REQUEST[] = { 0x11 };
 static const guint8 FS_SAMPLE_DELETE_DIR_REQUEST[] = { 0x12 };
 static const guint8 FS_SAMPLE_DELETE_FILE_REQUEST[] = { 0x20 };
 static const guint8 FS_SAMPLE_RENAME_FILE_REQUEST[] = { 0x21 };
+static const guint8 FS_SAMPLE_GET_FILE_INFO_FROM_PATH_REQUEST[] = { 0x22 };
 static const guint8 FS_SAMPLE_GET_FILE_INFO_FROM_HASH_AND_SIZE_REQUEST[] =
   { 0x23, 0, 0, 0, 0, 0, 0, 0, 0 };
 static const guint8 FS_SAMPLE_OPEN_FILE_READER_REQUEST[] = { 0x30 };
@@ -260,6 +255,7 @@ static const guint8 FS_RAW_CREATE_DIR_REQUEST[] = { 0x15 };
 static const guint8 FS_RAW_DELETE_DIR_REQUEST[] = { 0x16 };
 static const guint8 FS_RAW_DELETE_FILE_REQUEST[] = { 0x24 };
 static const guint8 FS_RAW_RENAME_FILE_REQUEST[] = { 0x25 };
+static const guint8 FS_RAW_GET_FILE_INFO_FROM_PATH_REQUEST[] = { 0x26 };
 static const guint8 FS_RAW_OPEN_FILE_READER_REQUEST[] = { 0x33 };
 static const guint8 FS_RAW_CLOSE_FILE_READER_REQUEST[] = { 0x34 };
 static const guint8 FS_RAW_READ_FILE_REQUEST[] =
@@ -296,6 +292,7 @@ static const struct fs_operations FS_SAMPLES_OPERATIONS = {
   .type_ext = "wav",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_samples_dir,
+  .file_exists = elektron_sample_file_exists,
   .print_item = elektron_print_smplrw,
   .mkdir = elektron_create_samples_dir,
   .delete = elektron_delete_samples_item,
@@ -317,6 +314,7 @@ static const struct fs_operations FS_RAW_ANY_OPERATIONS = {
   .type_ext = "raw",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_raw_dir,
+  .file_exists = elektron_raw_file_exists,
   .print_item = elektron_print_smplrw,
   .mkdir = elektron_create_raw_dir,
   .delete = elektron_delete_raw_item,
@@ -340,6 +338,7 @@ static const struct fs_operations FS_RAW_PRESETS_OPERATIONS = {
   .type_ext = "pst",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_raw_dir,
+  .file_exists = elektron_raw_file_exists,
   .print_item = elektron_print_smplrw,
   .mkdir = elektron_create_raw_dir,
   .delete = elektron_delete_raw_item,
@@ -356,7 +355,8 @@ static const struct fs_operations FS_RAW_PRESETS_OPERATIONS = {
 
 static const struct fs_operations FS_DATA_ANY_OPERATIONS = {
   .fs = FS_DATA_ALL,
-  .options = FS_OPTION_SORT_BY_ID | FS_OPTION_ID_AS_FILENAME,
+  .options = FS_OPTION_SORT_BY_ID | FS_OPTION_ID_AS_FILENAME |
+    FS_OPTION_SLOT_STORAGE,
   .name = "data",
   .type_ext = "data",
   .readdir = elektron_read_data_dir_any,
@@ -372,7 +372,7 @@ static const struct fs_operations FS_DATA_ANY_OPERATIONS = {
   .load = load_file,
   .save = save_file,
   .get_ext = elektron_get_dev_and_fs_ext,
-  .get_upload_path = elektron_get_upload_path_data,
+  .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
 
@@ -398,7 +398,7 @@ static const struct fs_operations FS_DATA_PRJ_OPERATIONS = {
   .load = load_file,
   .save = save_file,
   .get_ext = elektron_get_dev_and_fs_ext,
-  .get_upload_path = elektron_get_upload_path_data,
+  .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
 
@@ -424,7 +424,7 @@ static const struct fs_operations FS_DATA_SND_OPERATIONS = {
   .load = load_file,
   .save = save_file,
   .get_ext = elektron_get_dev_and_fs_ext,
-  .get_upload_path = elektron_get_upload_path_data,
+  .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
 
@@ -469,8 +469,8 @@ elektron_print_data (struct item_iterator *iter,
 {
   gchar *hsize = get_human_size (iter->item.size, FALSE);
   struct elektron_iterator_data *data = iter->data;
-  gchar *slot = iter->item.id > 0 ? elektron_get_id_as_slot (&iter->item,
-							     backend) : " -1";
+  gchar *slot = iter->item.id > 0 ?
+    elektron_get_id_as_slot (&iter->item, backend) : strdup (" -1");
   printf ("%c %04x %d %d %10s %s %s\n", iter->item.type, data->operations,
 	  data->has_valid_data, data->has_metadata, hsize, slot,
 	  iter->item.name);
@@ -1053,9 +1053,13 @@ elektron_tx_and_rx_timeout_no_cache (struct backend *backend,
 				     GByteArray * tx_msg, gint timeout)
 {
   ssize_t len;
+  guint16 seq;
   GByteArray *rx_msg;
   guint msg_type = tx_msg->data[4] | 0x80;
+  struct elektron_data *data = backend->data;
+  gint t = timeout < 0 ? BE_SYSEX_TIMEOUT_MS : timeout;
 
+  seq = data->seq;
   len = elektron_tx (backend, tx_msg);
   if (len < 0)
     {
@@ -1063,13 +1067,31 @@ elektron_tx_and_rx_timeout_no_cache (struct backend *backend,
       goto cleanup;
     }
 
-  rx_msg = elektron_rx (backend, timeout < 0 ? BE_SYSEX_TIMEOUT_MS : timeout);
-  if (rx_msg && rx_msg->data[4] != msg_type)
+  while (1)
     {
-      error_print ("Illegal message type in response\n");
-      free_msg (rx_msg);
-      rx_msg = NULL;
-      goto cleanup;
+      rx_msg = elektron_rx (backend, t);
+      if (!rx_msg)
+	{
+	  break;
+	}
+
+      guint16 exp_seq = be16toh (*((guint16 *) & rx_msg->data[2]));
+      if (seq != exp_seq)
+	{
+	  error_print ("Unexpected sequence in response. Skipping...\n");
+	  free_msg (rx_msg);
+	  continue;
+	}
+
+      if (rx_msg->data[4] != msg_type)
+	{
+	  error_print ("Illegal message type in response. Skipping...\n");
+	  free_msg (rx_msg);
+	  rx_msg = NULL;
+	  break;
+	}
+
+      break;
     }
 
 cleanup:
@@ -1130,10 +1152,7 @@ static enum item_type
 elektron_get_path_type (struct backend *backend, const gchar * path,
 			fs_init_iter_func init_iter)
 {
-  gchar *name_copy;
-  gchar *parent_copy;
-  gchar *name;
-  gchar *parent;
+  gchar *dir, *name;
   enum item_type res;
   struct item_iterator iter;
 
@@ -1142,12 +1161,10 @@ elektron_get_path_type (struct backend *backend, const gchar * path,
       return ELEKTROID_DIR;
     }
 
-  name_copy = strdup (path);
-  parent_copy = strdup (path);
-  name = basename (name_copy);
-  parent = dirname (parent_copy);
+  name = g_path_get_basename (path);
+  dir = g_path_get_dirname (path);
   res = ELEKTROID_NONE;
-  if (!init_iter (backend, &iter, parent))
+  if (!init_iter (backend, &iter, dir))
     {
       while (!next_item_iterator (&iter))
 	{
@@ -1160,8 +1177,8 @@ elektron_get_path_type (struct backend *backend, const gchar * path,
       free_item_iterator (&iter);
     }
 
-  g_free (name_copy);
-  g_free (parent_copy);
+  g_free (name);
+  g_free (dir);
   return res;
 }
 
@@ -1169,9 +1186,18 @@ static gint
 elektron_read_common_dir (struct backend *backend,
 			  struct item_iterator *iter, const gchar * dir,
 			  const guint8 msg[], int size,
-			  fs_init_iter_func init_iter, enum elektron_fs fs)
+			  fs_init_iter_func init_iter, enum elektron_fs fs,
+			  fs_file_exists file_exists)
 {
   GByteArray *tx_msg, *rx_msg = NULL;
+  gboolean is_file = file_exists (backend, dir);
+
+  usleep (BE_REST_TIME_US);
+
+  if (is_file)
+    {
+      return -ENOTDIR;
+    }
 
   tx_msg = elektron_new_msg_path (msg, size, dir);
   if (!tx_msg)
@@ -1203,7 +1229,8 @@ elektron_read_samples_dir (struct backend *backend,
   return elektron_read_common_dir (backend, iter, dir,
 				   FS_SAMPLE_READ_DIR_REQUEST,
 				   sizeof (FS_SAMPLE_READ_DIR_REQUEST),
-				   elektron_read_samples_dir, FS_SAMPLES);
+				   elektron_read_samples_dir, FS_SAMPLES,
+				   elektron_sample_file_exists);
 }
 
 static gint
@@ -1213,7 +1240,8 @@ elektron_read_raw_dir (struct backend *backend, struct item_iterator *iter,
   return elektron_read_common_dir (backend, iter, dir,
 				   FS_RAW_READ_DIR_REQUEST,
 				   sizeof (FS_RAW_READ_DIR_REQUEST),
-				   elektron_read_raw_dir, FS_RAW_ALL);
+				   elektron_read_raw_dir, FS_RAW_ALL,
+				   elektron_raw_file_exists);
 }
 
 static gint
@@ -1317,8 +1345,8 @@ elektron_move_common_item (struct backend *backend, const gchar * src,
 	{
 	  while (!next_item_iterator (&iter) && !res)
 	    {
-	      src_plus = chain_path (src, iter.item.name);
-	      dst_plus = chain_path (dst, iter.item.name);
+	      src_plus = path_chain (PATH_INTERNAL, src, iter.item.name);
+	      dst_plus = path_chain (PATH_INTERNAL, dst, iter.item.name);
 	      res = elektron_move_common_item (backend, src_plus, dst_plus,
 					       init_iter, mv, mkdir, rmdir);
 	      free (src_plus);
@@ -1365,8 +1393,7 @@ elektron_path_common (struct backend *backend, const gchar * path,
   else
     {
       res = -EPERM;
-      error_print ("%s (%s)\n", backend_strerror (backend, res),
-		   elektron_get_msg_string (rx_msg));
+      debug_print (1, "Error: %s\n", elektron_get_msg_string (rx_msg));
     }
   free_msg (rx_msg);
 
@@ -1392,11 +1419,36 @@ elektron_delete_samples_dir (struct backend *backend, const gchar * path)
 static gchar *
 elektron_add_ext_to_mc_snd (const gchar * path)
 {
-  gchar *path_with_ext = malloc (PATH_MAX);
-  snprintf (path_with_ext, PATH_MAX, "%s%s", path, ".mc-snd");
+  gchar *path_with_ext;
+  GString *str = g_string_new (path);
+  g_string_append (str, ".mc-snd");
+  path_with_ext = str->str;
+  g_string_free (str, FALSE);
   return path_with_ext;
 }
 
+static gboolean
+elektron_sample_file_exists (struct backend *backend, const gchar * path)
+{
+  gint res = elektron_path_common (backend, path,
+				   FS_SAMPLE_GET_FILE_INFO_FROM_PATH_REQUEST,
+				   sizeof
+				   (FS_SAMPLE_GET_FILE_INFO_FROM_PATH_REQUEST));
+  return res == 0;
+}
+
+static gboolean
+elektron_raw_file_exists (struct backend *backend, const gchar * path)
+{
+  gchar *name_with_ext = elektron_add_ext_to_mc_snd (path);
+  gint res = elektron_path_common (backend, path,
+				   FS_RAW_GET_FILE_INFO_FROM_PATH_REQUEST,
+				   sizeof
+				   (FS_RAW_GET_FILE_INFO_FROM_PATH_REQUEST));
+  g_free (name_with_ext);
+  return res == 0;
+}
+
 static gint
 elektron_delete_raw (struct backend *backend, const gchar * path)
 {
@@ -1485,10 +1537,9 @@ elektron_delete_common_item (struct backend *backend, const gchar * path,
 	  res = 0;
 	  while (!res && !next_item_iterator (&iter))
 	    {
-	      new_path = chain_path (path, iter.item.name);
-	      res = res
-		|| elektron_delete_common_item (backend, new_path,
-						init_iter, rmdir, rm);
+	      new_path = path_chain (PATH_INTERNAL, path, iter.item.name);
+	      res = res || elektron_delete_common_item (backend, new_path,
+							init_iter, rmdir, rm);
 	      free (new_path);
 	    }
 	  free_item_iterator (&iter);
@@ -1760,7 +1811,7 @@ elektron_download_smplrw (struct backend *backend, const gchar * path,
 	  sample_info->loopstart =
 	    be32toh (elektron_sample_header->loopstart);
 	  sample_info->loopend = be32toh (elektron_sample_header->loopend);
-	  sample_info->looptype = be32toh (elektron_sample_header->looptype);
+	  sample_info->looptype = elektron_sample_header->looptype;	// For some reason, this is already in the required format.
 	  sample_info->samplerate = be32toh (elektron_sample_header->samplerate);	//In the case of the RAW filesystem is not used and it is harmless.
 	  sample_info->channels = 1;
 	  sample_info->bitdepth = 16;
@@ -2037,7 +2088,7 @@ elektron_configure_device (struct backend *backend, guint8 id)
     }
   else
     {
-      devices_filename = get_expanded_dir (CONF_DIR DEVICES_FILE);
+      devices_filename = get_user_dir (CONF_DIR DEVICES_FILE);
     }
 
   if (!json_parser_load_from_file (parser, devices_filename, &error))
@@ -2220,6 +2271,8 @@ elektron_handshake (struct backend *backend)
       return -ENODEV;
     }
 
+  usleep (BE_REST_TIME_US);
+
   tx_msg = elektron_new_msg (SOFTWARE_VERSION_REQUEST,
 			     sizeof (SOFTWARE_VERSION_REQUEST));
   rx_msg = elektron_tx_and_rx (backend, tx_msg);
@@ -2233,6 +2286,8 @@ elektron_handshake (struct backend *backend)
   snprintf (backend->version, LABEL_MAX, "%s", (gchar *) & rx_msg->data[10]);
   free_msg (rx_msg);
 
+  usleep (BE_REST_TIME_US);
+
   if (debug_level > 1)
     {
       tx_msg = elektron_new_msg (DEVICEUID_REQUEST,
@@ -2243,6 +2298,8 @@ elektron_handshake (struct backend *backend)
 	  debug_print (1, "UID: %x\n", *((guint32 *) & rx_msg->data[5]));
 	  free_msg (rx_msg);
 	}
+
+      usleep (BE_REST_TIME_US);
     }
 
   snprintf (backend->description, LABEL_MAX, "%s", overbridge_name);
@@ -2355,15 +2412,18 @@ not_found:
 static gchar *
 elektron_add_prefix_to_path (const gchar * dir, const gchar * prefix)
 {
-  gchar *full = malloc (PATH_MAX);
+  gchar *full;
 
   if (prefix)
     {
-      snprintf (full, PATH_MAX, "%s%s", prefix, dir);
+      GString *str = g_string_new (NULL);
+      g_string_append_printf (str, "%s%s", prefix, dir);
+      full = str->str;
+      g_string_free (str, FALSE);
     }
   else
     {
-      strcpy (full, dir);
+      full = strdup (dir);
     }
 
   return full;
@@ -2766,7 +2826,8 @@ elektron_download_data_prefix (struct backend *backend, const gchar * path,
 			       struct job_control *control,
 			       const gchar * prefix)
 {
-  gint res;
+  gint err;
+  guint id;
   guint32 seq;
   guint32 seqbe;
   guint32 jid;
@@ -2781,11 +2842,18 @@ elektron_download_data_prefix (struct backend *backend, const gchar * path,
   gboolean active;
   GByteArray *rx_msg;
   GByteArray *tx_msg;
-  gchar *path_w_prefix = elektron_add_prefix_to_path (path, prefix);
+  gchar *path_w_prefix;
 
-  res = elektron_open_datum (backend, path_w_prefix, &jid, O_RDONLY, 0);
+  err = common_slot_get_id_name_from_path (path, &id, NULL);
+  if (err)
+    {
+      return err;
+    }
+
+  path_w_prefix = elektron_add_prefix_to_path (path, prefix);
+  err = elektron_open_datum (backend, path_w_prefix, &jid, O_RDONLY, 0);
   g_free (path_w_prefix);
-  if (res)
+  if (err)
     {
       return -EIO;
     }
@@ -2794,7 +2862,7 @@ elektron_download_data_prefix (struct backend *backend, const gchar * path,
 
   jidbe = htobe32 (jid);
 
-  res = 0;
+  err = 0;
   seq = 0;
   last = 0;
   control->data = NULL;
@@ -2812,14 +2880,14 @@ elektron_download_data_prefix (struct backend *backend, const gchar * path,
       rx_msg = elektron_tx_and_rx (backend, tx_msg);
       if (!rx_msg)
 	{
-	  res = -EIO;
+	  err = -EIO;
 	  break;
 	}
 
       if (!elektron_get_msg_status (rx_msg))
 	{
-	  res = -EPERM;
-	  error_print ("%s (%s)\n", backend_strerror (backend, res),
+	  err = -EPERM;
+	  error_print ("%s (%s)\n", backend_strerror (backend, err),
 		       elektron_get_msg_string (rx_msg));
 	  free_msg (rx_msg);
 	  break;
@@ -2909,34 +2977,29 @@ elektron_get_download_name (struct backend *backend,
 {
   gint32 id;
   gint ret;
-  const gchar *src_dir;
-  gchar *namec, *name, *src_dirc;
+  gchar *dir, *name;
   struct item_iterator *iter;
 
-  namec = g_strdup (src_path);
-  name = basename (namec);
-
   if (ops->fs == FS_SAMPLES || ops->fs == FS_RAW_ALL
       || ops->fs == FS_RAW_PRESETS)
     {
-      name = g_strdup (name);
-      goto cleanup;
+      return g_path_get_basename (src_path);
     }
 
   iter = malloc (sizeof (struct item_iterator));
-  src_dirc = g_strdup (src_path);
-  src_dir = dirname (src_dirc);
-  ret = ops->readdir (backend, iter, src_dir);
-  g_free (src_dirc);
+  dir = g_path_get_dirname (src_path);
+  ret = ops->readdir (backend, iter, dir);
+  g_free (dir);
   if (ret)
     {
-      name = NULL;
-      goto cleanup;
+      return NULL;
     }
 
+  name = g_path_get_basename (src_path);
   id = atoi (name);
-  name = NULL;
+  g_free (name);
 
+  name = NULL;
   while (!next_item_iterator (iter))
     {
       if (iter->item.id == id)
@@ -2948,8 +3011,6 @@ elektron_get_download_name (struct backend *backend,
 
   free_item_iterator (iter);
 
-cleanup:
-  g_free (namec);
   return name;
 }
 
@@ -3023,15 +3084,14 @@ static gchar *
 elektron_get_upload_path_smplrw (struct backend *backend,
 				 const struct fs_operations *ops,
 				 const gchar * dst_dir,
-				 const gchar * src_path, gint32 * next_index)
+				 const gchar * src_path)
 {
-  gchar *path, *namec, *name, *aux;
+  gchar *path, *name, *aux;
 
-  namec = strdup (src_path);
-  name = basename (namec);
+  name = g_path_get_basename (src_path);
   remove_ext (name);
-  aux = chain_path (dst_dir, name);
-  g_free (namec);
+  aux = path_chain (PATH_INTERNAL, dst_dir, name);
+  g_free (name);
 
   if (ops->fs == FS_RAW_ALL || ops->fs == FS_RAW_PRESETS)
     {
@@ -3045,45 +3105,6 @@ elektron_get_upload_path_smplrw (struct backend *backend,
   return path;
 }
 
-static gchar *
-elektron_get_upload_path_data (struct backend *backend,
-			       const struct fs_operations *ops,
-			       const gchar * dst_dir,
-			       const gchar * src_path, gint32 * next_index)
-{
-  gchar *indexs, *path;
-  struct item_iterator *remote_iter = malloc (sizeof (struct item_iterator));
-  if (ops->readdir (backend, remote_iter, dst_dir))
-    {
-      return strdup (dst_dir);
-    }
-
-  if (remote_iter->item.id == *next_index)
-    {
-      (*next_index)++;
-    }
-  else
-    {
-      while (!next_item_iterator (remote_iter))
-	{
-	  if (remote_iter->item.id > *next_index)
-	    {
-	      break;
-	    }
-	  (*next_index)++;
-	}
-    }
-
-  indexs = malloc (PATH_MAX);
-  snprintf (indexs, PATH_MAX, "%d", *next_index);
-  path = chain_path (dst_dir, indexs);
-  g_free (indexs);
-
-  (*next_index)++;
-
-  return path;
-}
-
 // As Elektron devices provide their own file extension and the file content is
 // not just SysEx, it is not needed to indicate in the filename the type of
 // device, filesystem or id.
@@ -3094,8 +3115,8 @@ elektron_get_download_path (struct backend *backend,
 			    const gchar * dst_dir, const gchar * src_path,
 			    GByteArray * data)
 {
-  gchar *path, *src_pathc, *name, *dl_ext, *filename;
-  const gchar *src_fpath, *md_ext, *ext = get_ext (src_path);
+  gchar *path, *name, *dl_ext, *src_fpath;
+  const gchar *md_ext, *ext = get_ext (src_path);
 
   // Examples:
   // 0:/project0
@@ -3103,56 +3124,45 @@ elektron_get_download_path (struct backend *backend,
   // 0:/soundbanks/A/1/.metadata
   // 0:/loops/sample
 
-  src_pathc = strdup (src_path);
   if (ext && strcmp (ext, "metadata") == 0)
     {
-      src_fpath = dirname (src_pathc);
+      src_fpath = g_path_get_dirname (src_path);
       md_ext = ".metadata";
     }
   else
     {
-      src_fpath = src_pathc;
+      src_fpath = strdup (src_path);
       md_ext = "";
     }
 
   name = elektron_get_download_name (backend, ops, src_fpath);
+  g_free (src_fpath);
   if (name)
     {
+      GString *filename = g_string_new (NULL);
       dl_ext = ops->get_ext (backend, ops);
-      filename = malloc (PATH_MAX);
-      snprintf (filename, PATH_MAX, "%s.%s%s", name, dl_ext, md_ext);
-      path = chain_path (dst_dir, filename);
+      g_string_append_printf (filename, "%s.%s%s", name, dl_ext, md_ext);
+      path = path_chain (PATH_SYSTEM, dst_dir, filename->str);
       g_free (name);
       g_free (dl_ext);
-      g_free (filename);
+      g_string_free (filename, TRUE);
     }
   else
     {
       path = NULL;
     }
 
-  g_free (src_pathc);
   return path;
 }
 
-static void
-elektron_remove_slot_name_from_path (gchar * path)
-{
-  gchar *c = path + strlen (path) - 1;
-  while (*c != ':')
-    {
-      c--;
-    }
-  *c = 0;
-}
-
 static gint
 elektron_upload_data_prefix (struct backend *backend, const gchar * path,
 			     GByteArray * array,
 			     struct job_control *control,
 			     const gchar * prefix)
 {
-  gint res;
+  gint err;
+  guint id;
   guint32 seq;
   guint32 jid;
   guint32 crc;
@@ -3167,13 +3177,21 @@ elektron_upload_data_prefix (struct backend *backend, const gchar * path,
   guint32 total;
   GByteArray *rx_msg;
   GByteArray *tx_msg;
-  gchar *path_w_prefix = elektron_add_prefix_to_path (path, prefix);
-  elektron_remove_slot_name_from_path (path_w_prefix);	//The slot name is not used with Elektron devices
+  gchar *path_w_prefix;
 
-  res = elektron_open_datum (backend, path_w_prefix, &jid, O_WRONLY,
+  err = common_slot_get_id_name_from_path (path, &id, NULL);
+  if (err)
+    {
+      return err;
+    }
+
+  path_w_prefix = elektron_add_prefix_to_path (path, prefix);
+  common_remove_slot_name_from_path (path_w_prefix);	//The slot name is not used with Elektron devices
+
+  err = elektron_open_datum (backend, path_w_prefix, &jid, O_WRONLY,
 			     array->len);
   g_free (path_w_prefix);
-  if (res)
+  if (err)
     {
       goto end;
     }
@@ -3198,9 +3216,8 @@ elektron_upload_data_prefix (struct backend *backend, const gchar * path,
 
   while (offset < array->len && active)
     {
-      tx_msg =
-	elektron_new_msg (DATA_WRITE_PARTIAL_REQUEST,
-			  sizeof (DATA_WRITE_PARTIAL_REQUEST));
+      tx_msg = elektron_new_msg (DATA_WRITE_PARTIAL_REQUEST,
+				 sizeof (DATA_WRITE_PARTIAL_REQUEST));
       g_byte_array_append (tx_msg, (guint8 *) & jidbe, sizeof (guint32));
       aux32 = htobe32 (seq);
       g_byte_array_append (tx_msg, (guint8 *) & aux32, sizeof (guint32));
@@ -3226,7 +3243,7 @@ elektron_upload_data_prefix (struct backend *backend, const gchar * path,
       rx_msg = elektron_tx_and_rx (backend, tx_msg);
       if (!rx_msg)
 	{
-	  res = -EIO;
+	  err = -EIO;
 	  goto end;
 	}
 
@@ -3234,8 +3251,8 @@ elektron_upload_data_prefix (struct backend *backend, const gchar * path,
 
       if (!elektron_get_msg_status (rx_msg))
 	{
-	  res = -EPERM;
-	  error_print ("%s (%s)\n", backend_strerror (backend, res),
+	  err = -EPERM;
+	  error_print ("%s (%s)\n", backend_strerror (backend, err),
 		       elektron_get_msg_string (rx_msg));
 	  free_msg (rx_msg);
 	  break;
@@ -3274,10 +3291,10 @@ elektron_upload_data_prefix (struct backend *backend, const gchar * path,
 
   debug_print (2, "%d bytes sent\n", offset);
 
-  res = elektron_close_datum (backend, jid, O_WRONLY, array->len);
+  err = elektron_close_datum (backend, jid, O_WRONLY, array->len);
 
 end:
-  return res;
+  return err;
 }
 
 static gint
