@@ -32,12 +32,13 @@
 #define SDS_DATA_PACKET_CKSUM_POS 125
 #define SDS_DATA_PACKET_CKSUM_START 1
 #define SDS_BYTES_PER_WORD 3
-#define SDS_MAX_RETRIES 3
+#define SDS_MAX_RETRIES 5
 #define SDS_SPEC_TIMEOUT 20	//Timeout in the specs to consider no response when transmission is going on.
 #define SDS_SPEC_TIMEOUT_HANDSHAKE 2000	//Timeout in the specs to consider no response during the handshake.
 #define SDS_NO_SPEC_TIMEOUT 5000	//Timeout used when the specs indicate to wait indefinitely.
 #define SDS_NO_SPEC_TIMEOUT_TRY 1500	//Timeout for SDS extensions that might not be implemented.
-#define SDS_REST_TIME_DEFAULT 18000	//Rest time to not overwhelm the devices whn sending consecutive packets. Lower values cause an an E-Mu ESI-2000 to send corrupted packets.
+#define SDS_REST_TIME_DEFAULT 50000	//Rest time to not overwhelm the devices when sending consecutive packets. Lower values cause an an E-Mu ESI-2000 to send corrupted packets.s
+#define SDS_INCOMPLETE_PACKET_TIMEOUT 2000
 #define SDS_NO_SPEC_OPEN_LOOP_REST_TIME 200000
 #define SDS_SAMPLE_CHANNELS 1
 #define SDS_SAMPLE_NAME_MAX_LEN 127
@@ -67,10 +68,10 @@ sds_get_download_path (struct backend *backend,
 		       const gchar * src_path, GByteArray * data)
 {
   GByteArray *tx_msg, *rx_msg;
-  gchar *name = malloc (PATH_MAX);
-  gchar *src_path_copy = strdup (src_path);
-  gchar *filename = basename (src_path_copy);
-  gint index = atoi (filename);
+  GString *str = g_string_new (NULL);
+  gchar *path;
+  gchar *name = g_path_get_basename (src_path);
+  gint index = atoi (name);
   gboolean use_id = TRUE;
   struct sds_data *sds_data = backend->data;
 
@@ -88,7 +89,7 @@ sds_get_download_path (struct backend *backend,
       rx_msg = backend_tx_and_rx_sysex (backend, tx_msg, SDS_NO_SPEC_TIMEOUT);
       if (rx_msg)
 	{
-	  snprintf (name, PATH_MAX, "%s/%s.wav", dst_dir, &rx_msg->data[5]);
+	  g_string_append_printf (str, "%s.wav", &rx_msg->data[5]);
 	  free_msg (rx_msg);
 	  use_id = FALSE;
 	}
@@ -96,11 +97,13 @@ sds_get_download_path (struct backend *backend,
 
   if (use_id)
     {
-      snprintf (name, PATH_MAX, "%s/%03d.wav", dst_dir, index);
+      g_string_append_printf (str, "%03d.wav", index);
     }
 
-  g_free (src_path_copy);
-  return name;
+  g_free (name);
+  path = path_chain (PATH_SYSTEM, dst_dir, str->str);
+  g_string_free (str, TRUE);
+  return path;
 }
 
 static guint
@@ -317,6 +320,8 @@ sds_download_get_header (struct backend *backend, guint id)
   GByteArray *tx_msg, *rx_msg;
 
   tx_msg = sds_get_request_msg (id);
+  g_byte_array_append (tx_msg, SDS_WAIT, sizeof (SDS_WAIT));	//We add a WAIT packet.
+  tx_msg->data[11] = 0;
   rx_msg = backend_tx_and_rx_sysex (backend, tx_msg, SDS_NO_SPEC_TIMEOUT);
 
   if (rx_msg && rx_msg->len == sizeof (SDS_DUMP_HEADER)
@@ -339,7 +344,7 @@ sds_download_try (struct backend *backend, const gchar * path,
     retries, packets, packet, exp_packet, rx_packets;
   gint16 sample;
   GByteArray *tx_msg, *rx_msg;
-  gchar *path_copy, *index;
+  gchar *name;
   guint8 *dataptr;
   gboolean active, first;
   gboolean last_packet_ack;
@@ -347,26 +352,42 @@ sds_download_try (struct backend *backend, const gchar * path,
   struct sysex_transfer transfer;
   struct sds_data *sds_data = backend->data;
 
-  path_copy = strdup (path);
-  index = basename (path_copy);
-  id = atoi (index);
-  g_free (path_copy);
+  name = g_path_get_basename (path);
+  id = atoi (name);
+  g_free (name);
 
   debug_print (1, "Sending dump request...\n");
   packet = 0;
 
-  g_mutex_lock (&backend->mutex);
-  backend_rx_drain (backend);
-  g_mutex_unlock (&backend->mutex);
-
-  rx_msg = sds_download_get_header (backend, id);
-  if (!rx_msg)
+  retries = 0;
+  while (1)
     {
-      err = -EIO;
-      goto end;
-    }
+      g_mutex_lock (&control->mutex);
+      active = control->active;
+      g_mutex_unlock (&control->mutex);
 
-  usleep (sds_data->rest_time);
+      if (!active)
+	{
+	  err = -ECANCELED;
+	  goto end;
+	}
+
+      g_mutex_lock (&backend->mutex);
+      backend_rx_drain (backend);
+      g_mutex_unlock (&backend->mutex);
+
+      rx_msg = sds_download_get_header (backend, id);
+      if (rx_msg)
+	{
+	  break;
+	}
+      retries++;
+      if (retries == SDS_MAX_RETRIES)
+	{
+	  err = -EIO;
+	  goto end;
+	}
+    }
 
   sample_info = malloc (sizeof (struct sample_info));
   if (sds_get_download_info (rx_msg, sample_info, &words, &word_size,
@@ -407,6 +428,7 @@ sds_download_try (struct backend *backend, const gchar * path,
       if (retries == SDS_MAX_RETRIES)
 	{
 	  debug_print (1, "Too many retries\n");
+	  err = -EIO;
 	  break;
 	}
 
@@ -426,19 +448,22 @@ sds_download_try (struct backend *backend, const gchar * path,
 	  err = backend_tx (backend, tx_msg);
 	  goto end;
 	}
-      else
+
+      if (last_packet_ack)
 	{
-	  transfer.raw = tx_msg;
-	  transfer.timeout = SDS_NO_SPEC_TIMEOUT;
-	  err = backend_tx_and_rx_sysex_transfer (backend, &transfer, FALSE);
-	  if (err == -ECANCELED)
-	    {
-	      break;
-	    }
-	  rx_msg = transfer.raw;
+	  sds_download_inc_packet (&first, &packet);
 	}
 
-      if (!rx_msg)
+      g_byte_array_append (tx_msg, SDS_WAIT, sizeof (SDS_WAIT));
+      tx_msg->data[10] = (packet) % 0x80;
+      transfer.raw = tx_msg;
+      transfer.timeout = SDS_INCOMPLETE_PACKET_TIMEOUT;	//This is enough to detect incomplete packets.
+      err = backend_tx_and_rx_sysex_transfer (backend, &transfer, FALSE);
+      if (err == -ECANCELED)
+	{
+	  break;
+	}
+      else if (err == -ETIMEDOUT)
 	{
 	  debug_print (2,
 		       "Packet not received. Remaining packets: %d; remaining samples: %d\n",
@@ -449,45 +474,42 @@ sds_download_try (struct backend *backend, const gchar * path,
 	      debug_print (2,
 			   "Skipping last packet as it has only one sample...\n");
 	      rx_packets++;
+
+	      //We cancel the upload.
+	      usleep (sds_data->rest_time);
+	      sds_tx_handshake (backend, SDS_CANCEL, packet % 0x80);
+	      usleep (sds_data->rest_time);
+
 	      err = 0;
 	      goto end;
 	    }
-	  err = (rx_packets == packets - 1) ? -EBADMSG : -EINVAL;
-	  sds_download_inc_packet (&first, &packet);
-	  break;
+	  rx_msg = NULL;
+	  goto retry;
+	}
+      else
+	{
+	  rx_msg = transfer.raw;
 	}
 
       if (rx_msg->len != SDS_DATA_PACKET_LEN)
 	{
-	  debug_print (2, "Invalid length. Stopping...\n");
-	  free_msg (rx_msg);
-	  err = -EBADMSG;
-	  break;
+	  debug_print (2, "Invalid length\n");
+	  goto retry;
 	}
 
       guint exp_packet_id = exp_packet % 0x80;
       if (rx_msg->data[4] != exp_packet_id)
 	{
-	  debug_print (2, "Invalid packet number. Stopping...\n");
-	  free_msg (rx_msg);
-	  err = -EINVAL;
-	  break;
-	}
-
-      if (last_packet_ack)
-	{
-	  sds_download_inc_packet (&first, &packet);
+	  debug_print (2, "Invalid packet number (%d != %d)\n",
+		       rx_msg->data[4], exp_packet_id);
+	  goto retry;
 	}
 
       if (sds_checksum (rx_msg->data) !=
 	  rx_msg->data[SDS_DATA_PACKET_CKSUM_POS])
 	{
-	  debug_print (2, "Invalid cksum. Retrying...\n");
-	  free_msg (rx_msg);
-	  last_packet_ack = FALSE;
-	  usleep (sds_data->rest_time);
-	  retries++;
-	  continue;
+	  debug_print (2, "Invalid cksum\n");
+	  goto retry;
 	}
 
       exp_packet++;
@@ -517,7 +539,18 @@ sds_download_try (struct backend *backend, const gchar * path,
 
       free_msg (rx_msg);
 
+      continue;
+
+    retry:
+      debug_print (2, "Retrying packet...\n");
+      if (rx_msg)
+	{
+	  free_msg (rx_msg);
+	}
+      last_packet_ack = FALSE;
       usleep (sds_data->rest_time);
+      retries++;
+      continue;
     }
 
   free_msg (tx_msg);
@@ -531,10 +564,11 @@ end:
   else
     {
       debug_print (1, "Cancelling SDS download...\n");
-      usleep (sds_data->rest_time);
       sds_tx_handshake (backend, SDS_CANCEL, packet % 0x80);
     }
 
+  usleep (sds_data->rest_time);
+
   return err;
 }
 
@@ -676,7 +710,7 @@ sds_rename (struct backend *backend, const gchar * src, const gchar * dst)
   GByteArray *tx_msg, *rx_msg;
   guint id;
   gint err;
-  gchar *name, *dstcpy;
+  gchar *name;
   debug_print (1, "Sending rename request...\n");
   err = common_slot_get_id_name_from_path (src, &id, NULL);
   if (err)
@@ -688,8 +722,7 @@ sds_rename (struct backend *backend, const gchar * src, const gchar * dst)
   backend_rx_drain (backend);
   g_mutex_unlock (&backend->mutex);
 
-  dstcpy = strdup (dst);
-  name = basename (dstcpy);
+  name = g_path_get_basename (dst);
   tx_msg = sds_get_rename_sample_msg (id, name);
   err = -ENOSYS;
   rx_msg = backend_tx_and_rx_sysex (backend, tx_msg, SDS_NO_SPEC_TIMEOUT);
@@ -699,7 +732,7 @@ sds_rename (struct backend *backend, const gchar * src, const gchar * dst)
       free_msg (rx_msg);
     }
 
-  g_free (dstcpy);
+  g_free (name);
   return err;
 }
 
@@ -723,7 +756,7 @@ sds_upload (struct backend *backend, const gchar * path, GByteArray * input,
 
   if (common_slot_get_id_name_from_path (path, &id, &name))
     {
-      return -EBADSLT;
+      return -EINVAL;
     }
 
   g_mutex_lock (&backend->mutex);
@@ -743,7 +776,7 @@ sds_upload (struct backend *backend, const gchar * path, GByteArray * input,
   packets = ceil (words / (double) words_per_packet);
 
   tx_msg = sds_get_dump_msg (id, words, sample_info, bits);
-  //The first timeout should be SDS_SPEC_TIMEOUT_HANDSHAKE (2 s) buit it is not enough sometimes.
+  //The first timeout should be SDS_SPEC_TIMEOUT_HANDSHAKE (2 s) but it is not enough sometimes.
   err = sds_tx_and_wait_ack (backend, tx_msg, 0, SDS_NO_SPEC_TIMEOUT,
 			     SDS_NO_SPEC_TIMEOUT);
   if (err == -ENOMSG)
@@ -770,6 +803,11 @@ sds_upload (struct backend *backend, const gchar * path, GByteArray * input,
   frame = (gint16 *) input->data;
   while (packet < packets && active)
     {
+      if (retries)
+	{
+	  usleep (sds_data->rest_time);
+	}
+
       if (retries == SDS_MAX_RETRIES)
 	{
 	  debug_print (1, "Too many retries\n");
@@ -808,14 +846,15 @@ sds_upload (struct backend *backend, const gchar * path, GByteArray * input,
 	}
       else if (err == -EINVAL)
 	{
-	  debug_print (2, "Unexpectd packet number. Continuing...\n");
-	  goto end;
+	  debug_print (2, "Unexpected packet number. Retrying...\n");
+	  retries++;
+	  continue;
 	}
       else if (err == -ETIMEDOUT)
 	{
-	  debug_print (2, "No response. Continuing in open loop...\n");
-	  open_loop = TRUE;
-	  err = 0;
+	  debug_print (2, "No response. Retrying...\n");
+	  retries++;
+	  continue;
 	}
       else if (err == -ECANCELED)
 	{
@@ -1029,12 +1068,17 @@ sds_handshake (struct backend *backend)
   GByteArray *tx_msg, *rx_msg;
   struct sds_data *sds_data = g_malloc (sizeof (struct sds_data));
 
+  //We cancel anything that might be running.
+  usleep (SDS_REST_TIME_DEFAULT);
+  sds_tx_handshake (backend, SDS_CANCEL, 0);
+  usleep (SDS_REST_TIME_DEFAULT);
+
   //Elektron devices support SDS so we need to be sure it is not.
   rx_msg = elektron_ping (backend);
   if (rx_msg)
     {
-      free_msg (rx_msg);
-      g_free (backend->data);	//This is filled up by elektron_ping.
+      free_msg (rx_msg);	//This is filled up by elektron_ping.
+      g_free (backend->data);
       return -ENODEV;
     }
 
@@ -1046,9 +1090,9 @@ sds_handshake (struct backend *backend)
   //Numbers higher than 1500 make an E-Mu ESI-2000 crash when entering into the 'MIDI SAMPLE DUMP' menu but the actual limit is unknown.
   tx_msg = sds_get_dump_msg (1000, 0, NULL, 16);
   //In case we receive an ACK, NAK or CANCEL, there is a MIDI SDS device listening.
-  err = sds_tx_and_wait_ack (backend, tx_msg, 0, SDS_SPEC_TIMEOUT_HANDSHAKE,
+  err = sds_tx_and_wait_ack (backend, tx_msg, 0,
+			     SDS_SPEC_TIMEOUT_HANDSHAKE,
 			     SDS_NO_SPEC_TIMEOUT_TRY);
-
   if (err && err != -EBADMSG && err != -ECANCELED)
     {
       return -ENODEV;
