@@ -21,8 +21,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <libgen.h>
+#if defined(__linux__)
 #include <signal.h>
+#endif
 #include <stdint.h>
 #include <inttypes.h>
 #include <stddef.h>
@@ -47,7 +48,7 @@ cli_get_path (gchar * device_path)
   gchar *path = device_path;
   gint i = 0;
 
-  while (path[0] != '/' && i < len)
+  while (*path != '/' && i < len)
     {
       path++;
       i++;
@@ -66,7 +67,7 @@ cli_ld ()
   for (i = 0; i < devices->len; i++)
     {
       device = g_array_index (devices, struct backend_system_device, i);
-      printf ("%d: %s %s\n", i, device.id, device.name);
+      printf ("%d: id: %s; name: %s\n", i, device.id, device.name);
     }
 
   g_array_free (devices, TRUE);
@@ -81,14 +82,14 @@ cli_connect (const gchar * device_path)
   struct backend_system_device device;
   GArray *devices = backend_get_system_devices ();
 
-  if (!devices->len)
+  if (!devices->len || id >= devices->len)
     {
       error_print ("Invalid device %d\n", id);
       return -ENODEV;
     }
 
   device = g_array_index (devices, struct backend_system_device, id);
-  err = connector_init_backend (&backend, device.id, connector, NULL);
+  err = connector_init_backend (&backend, &device, connector, NULL);
   g_array_free (devices, TRUE);
 
   if (!err && fs)
@@ -394,6 +395,8 @@ cli_df (int argc, gchar * argv[], int *optind)
 	  g_free (diff);
 	  g_free (free);
 	}
+
+      sleep (BE_REST_TIME_US);
     }
 
   return err;
@@ -508,11 +511,10 @@ end:
 static int
 cli_upload (int argc, gchar * argv[], int *optind)
 {
-  const gchar *dst_dir;
+  const gchar *dst_path;
   gchar *src_path, *device_dst_path, *upload_path;
   gint err;
   GByteArray *array;
-  gint32 index = 1;
 
   if (*optind == argc)
     {
@@ -542,10 +544,10 @@ cli_upload (int argc, gchar * argv[], int *optind)
       return err;
     }
 
-  dst_dir = cli_get_path (device_dst_path);
+  dst_path = cli_get_path (device_dst_path);
 
-  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_dir,
-					 src_path, &index);
+  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
+					 src_path);
 
   array = g_byte_array_new ();
   control.active = TRUE;
@@ -699,6 +701,7 @@ set_conn_fs_op_from_command (const gchar * cmd)
   return 0;
 }
 
+#if defined(__linux__)
 static void
 cli_end (int sig)
 {
@@ -710,6 +713,7 @@ cli_end (int sig)
   sysex_transfer.active = FALSE;
   g_mutex_unlock (&sysex_transfer.mutex);
 }
+#endif
 
 int
 main (int argc, gchar * argv[])
@@ -718,6 +722,7 @@ main (int argc, gchar * argv[])
   gint err;
   gchar *command;
   gint vflg = 0, errflg = 0;
+#if defined(__linux__)
   struct sigaction action;
 
   action.sa_handler = cli_end;
@@ -727,6 +732,7 @@ main (int argc, gchar * argv[])
   sigaction (SIGQUIT, &action, NULL);
   sigaction (SIGINT, &action, NULL);
   sigaction (SIGHUP, &action, NULL);
+#endif
 
   while ((c = getopt (argc, argv, "v")) != -1)
     {
@@ -758,7 +764,7 @@ main (int argc, gchar * argv[])
   if (errflg > 0)
     {
       fprintf (stderr, "%s\n", PACKAGE_STRING);
-      gchar *exec_name = basename (argv[0]);
+      gchar *exec_name = g_path_get_basename (argv[0]);
       fprintf (stderr, "Usage: %s [options] command\n", exec_name);
       exit (EXIT_FAILURE);
     }
