@@ -20,7 +20,6 @@
 
 #include <dirent.h>
 #include <errno.h>
-#include <libgen.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -68,20 +67,19 @@ local_get_download_path (struct backend *backend,
 			 const gchar * dst_dir, const gchar * src_path,
 			 GByteArray * content)
 {
-  gchar *src_pathc = strdup (src_path);
-  gchar *path = malloc (PATH_MAX);
-  gchar *filename = basename (src_pathc);
-  remove_ext (filename);
-  snprintf (path, PATH_MAX, "%s/%s.wav", dst_dir, filename);
-  g_free (src_pathc);
+  gchar *name = g_path_get_basename (src_path);
+  GString *name_with_ext = g_string_new (NULL);
+  remove_ext (name);
+  g_string_append_printf (name_with_ext, "%s.%s", name, ops->type_ext);
+  gchar *path = path_chain (PATH_SYSTEM, dst_dir, name_with_ext->str);
+  g_string_free (name_with_ext, TRUE);
   return path;
 }
 
 static gchar *
 local_get_upload_path (struct backend *backend,
 		       const struct fs_operations *ops,
-		       const gchar * dst_dir,
-		       const gchar * src_path, gint32 * next_index)
+		       const gchar * dst_dir, const gchar * src_path)
 {
   return local_get_download_path (backend, ops, dst_dir, src_path, NULL);
 }
@@ -89,29 +87,8 @@ local_get_upload_path (struct backend *backend,
 gint
 local_mkdir (struct backend *backend, const gchar * name)
 {
-  DIR *dir;
-  gint res = 0;
-  gchar *dup;
-  gchar *parent;
-
-  dup = strdup (name);
-  parent = dirname (dup);
-
-  dir = opendir (parent);
-  if (dir)
-    {
-      closedir (dir);
-    }
-  else
-    {
-      res = local_mkdir (backend, parent);
-      if (res)
-	{
-	  goto cleanup;
-	}
-    }
-
-  if (mkdir (name, 0755) == 0 || errno == EEXIST)
+  gint res = g_mkdir_with_parents (name, 0755);
+  if (res == 0 || errno == EEXIST)
     {
       res = 0;
     }
@@ -121,8 +98,6 @@ local_mkdir (struct backend *backend, const gchar * name)
       res = -errno;
     }
 
-cleanup:
-  g_free (dup);
   return res;
 }
 
@@ -144,7 +119,7 @@ local_delete (struct backend *backend, const gchar * path)
 	    {
 	      continue;
 	    }
-	  new_path = chain_path (path, dirent->d_name);
+	  new_path = path_chain (PATH_SYSTEM, path, dirent->d_name);
 	  local_delete (backend, new_path);
 	  free (new_path);
 	}
@@ -193,7 +168,7 @@ local_next_dentry (struct item_iterator *iter)
 	  continue;
 	}
 
-      full_path = chain_path (data->path, dirent->d_name);
+      full_path = path_chain (PATH_SYSTEM, data->path, dirent->d_name);
       if (stat (full_path, &st))
 	{
 	  free (full_path);
@@ -329,6 +304,12 @@ local_sample_load_441_16_mono (const gchar * path, GByteArray * sample,
   return local_sample_load_custom (path, sample, control, &sample_params);
 }
 
+static gboolean
+local_file_exists (struct backend *backend, const gchar * path)
+{
+  return access (path, F_OK) == 0;
+}
+
 const struct fs_operations FS_LOCAL_OPERATIONS = {
   .fs = 0,
   .options = FS_OPTION_SORT_BY_NAME | FS_OPTION_AUDIO_PLAYER |
@@ -337,6 +318,7 @@ const struct fs_operations FS_LOCAL_OPERATIONS = {
   .gui_name = "localhost",
   .gui_icon = BE_FILE_ICON_WAVE,
   .readdir = local_read_dir,
+  .file_exists = local_file_exists,
   .mkdir = local_mkdir,
   .delete = local_delete,
   .rename = local_rename,
@@ -362,6 +344,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
   .gui_name = "WAV 48 KHz 16 bits stereo",
   .gui_icon = BE_FILE_ICON_WAVE,
   .readdir = local_read_dir,
+  .file_exists = local_file_exists,
   .mkdir = local_mkdir,
   .delete = local_delete,
   .rename = local_rename,
@@ -385,6 +368,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
   .gui_name = "WAV 48 KHz 16 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .readdir = local_read_dir,
+  .file_exists = local_file_exists,
   .mkdir = local_mkdir,
   .delete = local_delete,
   .rename = local_rename,
@@ -409,6 +393,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
   .gui_name = "WAV 44.1 KHz 16 bits stereo",
   .gui_icon = BE_FILE_ICON_WAVE,
   .readdir = local_read_dir,
+  .file_exists = local_file_exists,
   .mkdir = local_mkdir,
   .delete = local_delete,
   .rename = local_rename,
@@ -432,6 +417,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
   .gui_name = "WAV 44.1 KHz 16 bits mono",
   .gui_icon = BE_FILE_ICON_WAVE,
   .readdir = local_read_dir,
+  .file_exists = local_file_exists,
   .mkdir = local_mkdir,
   .delete = local_delete,
   .rename = local_rename,
@@ -472,5 +458,7 @@ system_init_backend (struct backend *backend, const gchar * id)
     FS_SAMPLES_LOCAL_441_16_STEREO | FS_SAMPLES_LOCAL_441_16_MONO;
   backend->fs_ops = FS_SYSTEM_OPERATIONS;
   snprintf (backend->name, LABEL_MAX, "%s", _("System"));
+  *backend->version = 0;
+  *backend->description = 0;
   return 0;
 }
