@@ -24,6 +24,8 @@
 #include <errno.h>
 #include "sample.h"
 
+#define LOAD_BUFFER_LEN (8 * 1024)
+
 #define JUNK_CHUNK_ID "JUNK"
 #define SMPL_CHUNK_ID "smpl"
 
@@ -328,7 +330,8 @@ audio_multichannel_to_mono (gshort * input, gshort * output, gint size,
 static gint
 sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 		 struct job_control *control, GByteArray * sample,
-		 const struct sample_params *sample_params, guint * frames)
+		 const struct sample_params *sample_params, guint * frames,
+		 sample_load_cb cb, gpointer cb_data)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -562,8 +565,7 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
       if (control)
 	{
 	  g_mutex_lock (&control->mutex);
-	  set_job_control_progress_no_sync (control,
-					    f * 1.0 / sf_info.frames);
+	  cb (control, f * 1.0 / sf_info.frames, cb_data);
 	  active = control->active;
 	  g_mutex_unlock (&control->mutex);
 	}
@@ -577,7 +579,7 @@ sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 
       if (control->active)
 	{
-	  set_job_control_progress_no_sync (control, 1.0);
+	  cb (control, 1.0, cb_data);
 	}
       else
 	{
@@ -626,7 +628,26 @@ sample_load_from_array (GByteArray * wave, GByteArray * sample,
   data.pos = 0;
   data.array = wave;
   return sample_load_raw (&data, &G_BYTE_ARRAY_IO, control, sample,
-			  sample_params, frames);
+			  sample_params, frames,
+			  set_job_control_progress_no_sync, NULL);
+}
+
+gint
+sample_load_from_file_with_cb (const gchar * path, GByteArray * sample,
+			       struct job_control *control,
+			       const struct sample_params *sample_params,
+			       guint * frames, sample_load_cb cb,
+			       gpointer cb_data)
+{
+  FILE *file = fopen (path, "rb");
+  if (!file)
+    {
+      return -errno;
+    }
+  gint err = sample_load_raw (file, &FILE_IO, control, sample, sample_params,
+			      frames, cb, cb_data);
+  fclose (file);
+  return err;
 }
 
 gint
@@ -635,22 +656,17 @@ sample_load_from_file (const gchar * path, GByteArray * sample,
 		       const struct sample_params *sample_params,
 		       guint * frames)
 {
-  FILE *file = fopen (path, "rb");
-  if (!file)
-    {
-      return errno;
-    }
-  gint err = sample_load_raw (file, &FILE_IO, control, sample, sample_params,
-			      frames);
-  fclose (file);
-  return err;
+  return sample_load_from_file_with_cb (path, sample, control,
+					sample_params, frames,
+					set_job_control_progress_no_sync,
+					NULL);
 }
 
 static gboolean
 sample_is_mp3_supported ()
 {
+#if defined(__linux__)
   static char buffer[LABEL_MAX];
-
   sf_command (NULL, SFC_GET_LIB_VERSION, buffer, LABEL_MAX);
   debug_print (1, "libsndfile version: %s...\n", buffer);
   if (strverscmp (buffer, "libsndfile-1.1.0") >= 0)
@@ -659,6 +675,9 @@ sample_is_mp3_supported ()
     }
 
   return FALSE;
+#else
+  return TRUE;
+#endif
 }
 
 const gchar **
